% ----------------------------------------------------------------------------------------------%
%                                                                                               %
% DDDDDDDDDDD    RRRRRRRRRR     EEEEEEEEEE       AA       MMM         MMM    ZZZZZZZZ SSSSSSSSS %
% DDDDDDDDDDDD   RRRRRRRRRRR    EEEEEEEEEE       AA       MMM         MMM    ZZZZZZZZ SSSSSSSSS %
% DDD      DDD   RRR      RRR   EEE             AAAA      MMMM        MMM         ZZZ SSS       %
% DDD      DDD   RRR      RRR   EEE             AAAA      MMMMM      MMMM         ZZZ SSS       %
% DDD      DDD   RRR      RRR   EEE            AA  AA     MMMMM     MMMMM        ZZZ  SSS       %
% DDD      DDD   RRR      RRR   EEE            AA  AA     MMM MM   MM MMM        ZZZ  SSSS      %
% DDD      DDD   RRRRRRRRRRRR   EEEEEEEEEE    AA    AA    MMM  MMMMM  MMM --    ZZZ   SSSSSSSSS %
% DDD      DDD   RRRRRRRRRRRR   EEEEEEEEEE    AAAAAAAA    MMM   MMM   MMM --    ZZZ   SSSSSSSSS %
% DDD      DDD   RRR      RRR   EEE          AAA    AAA   MMM    M    MMM      ZZZ         SSSS %
% DDD      DDD   RRR      RRR   EEE          AAA    AAA   MMM         MMM      ZZZ          SSS %
% DDD      DDD   RRR      RRR   EEE         AAA      AAA  MMM         MMM     ZZZ           SSS %
% DDD      DDD   RRR      RRR   EEE         AAA      AAA  MMM         MMM     ZZZ           SSS %
% DDDDDDDDDDDD   RRR      RRR   EEEEEEEEEE  AAA      AAA  MMM         MMM    ZZZZZZZZZ SSSSSSSS %
% DDDDDDDDDDD    RRR      RRR   EEEEEEEEEE  AAA      AAA  MMM         MMM    ZZZZZZZZZ SSSSSSSS %
%                                                                                               %
% ----------------------------------------------------------------------------------------------%

% ---- DiffeRential Evolution Adaptive Metropolis algorithm with sampling from past states ---- %
%                                                                                               %
% DREAM runs multiple different chains simultaneously for global exploration, and automatically %
% tunes the scale and orientation of the proposal distribution using differential evolution.    %
% The algorithm maintains detailed balance and ergodicity and works well and efficient for a    %
% large range of problems, especially in the presence of high-dimensionality and                %
% multimodality. This version creates proposals from an archive of past states rather than      %
% the current position of the chains (as in DREAM)                                              %
%                                                                                               %
% DREAM_ZS developed by Jasper A. Vrugt and Cajo ter Braak                                      %
%                                                                                               %
% --------------------------------------------------------------------------------------------- %
%                                                                                               %
% SYNOPSIS:                                                                                     %
%                                                                                               %
%   [chain,output,fx,log_L] = DREAM_ZS(Func_name,DREAMPar,Par_info)                             %
%   [chain,output,fx,log_L] = DREAM_ZS(Func_name,DREAMPar,Par_info,Meas_info)                   %
%   [chain,output,fx,log_L] = DREAM_ZS(Func_name,DREAMPar,Par_info,Meas_info,options)           %
%   [chain,output,fx,log_L] = DREAM_ZS(Func_name,DREAMPar,Par_info,Meas_info,options,plug)      %
%                                                                                               %
% Input:    Func_name = name of the function ( = model ) that returns density of proposal       %
%           DREAMPar = structure with algorithmic / computatinal settings of DREAM              %
%           Par_info = structure with parameter ranges, prior distribution, boundary handling   %
%           Meas_info = optional structure with measurements to be evaluated against            %
%           options = optional structure with additional options                                %
%           plug = variable/vector/matrix/structure of additional arguments passed to function  %
%                                                                                               %
% Output:   chain = 3D array with chain trajectories, log-prior and log-likelihood values       %
%           output = structure with convergence properties, acceptance rate, CR values, etc.    %
%           fx = matrix with model simulations                                                  %
%           Z = matrix with historical archive of samples                                       %
%                                                                                               %
% --------------------------------------------------------------------------------------------- %
%                                                                                               %
% This algorithm has been described in:                                                         %
%                                                                                               %
%   Vrugt, J.A., Markov chain Monte Carlo simulation using the DREAM software package: Theory,  %
%       concepts, and MATLAB Implementation, Environmental Modelling & Software, 75, 273-316,   %
%       doi:10.1016/j.envsoft.2015.08.013.                                                      %
%   Sadegh, M., and J.A. Vrugt, Approximate Bayesian computation using Markov chain Monte       %
%       Carlo simulation: DREAM_(ABC), Water Resources Research, doi:10.1002/2014WR015386,      %
%       2014.                                                                                   %
%   Vrugt, J.A., and M. Sadegh, Toward diagnostic model calibration and evaluation:             %
%       Approximate Bayesian computation, Water Resources Research, 49, 43354345,              %
%       doi:10.1002/wrcr.20354, 2013.                                                           %
%   Laloy, E., and J.A. Vrugt, High-dimensional posterior exploration of hydrologic models      %
%       using multiple-try DREAM_(ZS) and high-performance computing, Water Resources Research, %
%       48, W01526, doi:10.1029/2011WR010608, 2012.                                             %
%   Vrugt, J.A., and C.J.F. ter Braak, DREAM_(D): An adaptive Markov chain Monte Carlo          %
%      simulation algorithm to solve discrete, noncontinuous, and combinatorial posterior       %
%      parameter estimation problems, Hydrology and Earth System Sciences, 15, 3701-3713,       %
%      doi:10.5194/hess-15-3701-2011, 2011.                                                     %
%   Vrugt, J.A., C.J.F. ter Braak, H.V. Gupta, and B.A. Robinson, Equifinality of formal        %
%       (DREAM) and informal (GLUE) Bayesian approaches in hydrologic modeling?, Stochastic     %
%       Environmental Research and Risk Assessment, 23(7), 1011-1026, 				            %
%       doi:10.1007/s00477-008-0274-y, 2009                                                     %
%   Vrugt, J.A., C.J.F. ter Braak, C.G.H. Diks, D. Higdon, B.A. Robinson, and J.M. Hyman,       %
%       Accelerating Markov chain Monte Carlo simulation by differential evolution with         %
%       self-adaptive randomized subspace sampling, International Journal of Nonlinear          %
%       Sciences and Numerical Simulation, 10(3), 271-288, 2009.                                %
%   Vrugt, J.A., C.J.F. ter Braak, M.P. Clark, J.M. Hyman, and B.A. Robinson, Treatment of      %
%      input uncertainty in hydrologic modeling: Doing hydrology backward with Markov chain     %
%      Monte Carlo simulation, Water Resources Research, 44, W00B09, doi:10.1029/2007WR006720,  %
%      2008.                                                                                    %
%                                                                                               %
% --------------------------------------------------------------------------------------------- %
%                               For more information please read:                               %
% --------------------------------------------------------------------------------------------- %
%                                                                                               %
%   Ter Braak, C.J.F., A Markov Chain Monte Carlo version of the genetic algorithm Differential %
%       Evolution: easy Bayesian computing for real parameter spaces, Stat. Comput., 16,        %
%       239 - 249, doi:10.1007/s11222-006-8769-1, 2006.                                         %
%   Vrugt, J.A., H.V. Gupta, W. Bouten and S. Sorooshian, A Shuffled Complex Evolution          %
%       Metropolis algorithm for optimization and uncertainty assessment of hydrologic model    %
%       parameters, Water Resour. Res., 39 (8), 1201, doi:10.1029/2002WR001642, 2003.           %
%   Ter Braak, C.J.F., and J.A. Vrugt, Differential Evolution Markov Chain with snooker updater %
%       and fewer chains, Statistics and Computing, 10.1007/s11222-008-9104-9, 2008.            %
%                                                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                                               %
% MATLAB code written by Jasper A. Vrugt, University of California Irvine, jasper@uci.edu      	%
%                                                                                               %
% Version 3.0: March 2016     Code as described in EMS (2016) paper                             %
%                                                                                               %
% --------------------------------------------------------------------------------------------- %

% --------------------------------------------------------------------------------------------- %
%                                                                                               %
%            Note: IF WORKING WITH INTEGER PROBLEMS, PLEASE USED DREAM_(DZS)              	    %
%                                                                                               %
% --------------------------------------------------------------------------------------------- %

%% Check:  http://faculty.sites.uci.edu/jasper
%% Papers: http://faculty.sites.uci.edu/jasper/publications/
%% Google Scholar: https://scholar.google.com/citations?user=zkNXecUAAAAJ&hl=nl

%% Different test examples
%% example 1: n-dimensional banana shaped Gaussian distribution
%% example 2: n-dimensional Gaussian distribution
%% example 3: n-dimensional multimodal mixture distribution
%% example 4: real-world example rainfall-runoff (hymod coded in MATLAB)
%% example 5: real-world example rainfall-runoff (hymod coded in FORTRAN)
%% example 6: rainfall-runoff with generalized log-likelihood function
%% example 7: HYDRUS-1D soil hydraulic model: multiplicative prior
%% example 8: 1D mixture distribution: Approximate Bayesian Computation
%% example 9: Rainfall-runoff model with spectral likelihood function
%% example 10: multimodel mixture distrbibution: multivariate prior
%% example 11: multivariate student t distribution
%% example 12: pedometrics problem involving variogram fitting
%% example 13: Nash-Cascade hydrograph
%% example 14: ABC inference for hydrologic model
%% example 15: ABC inference using 10 bivariate normal distributions
%% example 16: Hydrogeophysical inference
%% example 17: rainfall - runoff with inference discharge data error
%% example 18: informal likelihood (GLUE): Lotka-Volterra model
%% example 19: Bayesian Model Averaging
%% example 20: GLUE - limits of acceptability: Soil temperature modeling
%% example 21: GLUE - limits of acceptability: HYDRUS-1D model
%% example 22: GLUE - limits of acceptability: Nash-Cascade example
%% example 23: GLUE - limits of acceptability: SAC-SMA model
%% ------------------------------------------------------------------------

% Clear memory and close pool of matlab nodes (cores)
clear all; close all hidden; clc; warning off

% Which example to run?
example = 4;

%% ########################### DO NOT CHANGE ##############################

global DREAM_dir EXAMPLE_dir CONV_dir

%% Check whether running on PC or MAC/UNIX machine
if ispc, slash_dir = '\'; else slash_dir = '/'; end

%% Store working directory and subdirectory containing the files needed to run this example
DREAM_dir = pwd; EXAMPLE_dir = [pwd ,slash_dir,'example_' num2str(example)]; CONV_dir = [pwd ,slash_dir,'diagnostics'];

%% Add subdirectory to search path
addpath(EXAMPLE_dir); addpath(CONV_dir); addpath(DREAM_dir);

%% ########################################################################

%% NOW RUN EXAMPLE
evalstr = strcat('example_',num2str(example)); eval(evalstr);

%% Create a single matrix with values sampled by chains
ParSet = GenParSet(chain);

%% ########################################################################
%%      Func_name: Name of the function script of the model/function
%% ########################################################################
%%                        CASE STUDY DEPENDENT
%% ------------------------------------------------------------------------
%% Func_name                 % Name of the model function script (.m file)
%% ------------------------------------------------------------------------

%% ########################################################################
%%  DREAMPar: Computational setup DREAM and values algorithmic parameters
%% ########################################################################
%%                         CASE STUDY DEPENDENT
%% ------------------------------------------------------------------------
%% DREAMPar.d                        % Dimensionality target distribution
%% DREAMPar.T                        % Number of generations
%% DREAMPar.lik                      % Choice of likelihood function
%% ------------------------------------------------------------------------
%%                           DEFAULT VALUES
%% ------------------------------------------------------------------------
%% DREAMPar.N = 3;                   % Number of Markov chains
%% DREAMPar.nCR = 3                  % Number of crossover values
%% DREAMPar.delta = 3                % Number chain pairs for proposal
%% DREAMPar.lambda = 0.05            % Random error for ergodicity
%% DREAMPar.zeta = 0.05              % Randomization
%% DREAMPar.m0 = 10*d;               % Initialize size of external archive
%% DREAMPar.k = 10;                  % Appending rate of external archive
%% DREAMPar.pJumpRate_one = 0.2      % Probability of jumprate of 1
%% DREAMPar.pCR = 'yes'              % Adapt selection prob. crossover?
%% DREAMPar.thinning = 1             % Each Tth sample is stored
%% DREAMPar.GLUE = 10;               % GLUE likelihood parameter
%% DREAMPar.beta0 = 1;               % Scaling factor of built-in jump rate
%% ------------------------------------------------------------------------

%% ########################################################################
%%   Par_info: All information about the parameter space and prior
%% ########################################################################
%%                        CASE STUDY DEPENDENT
%% ------------------------------------------------------------------------
%% Par_info.initial          % Initial sampling distribution
%%                             ('uniform'/'latin'/'normal'/'prior')
%% Par_info.min              % If 'latin', min parameter values
%% Par_info.max              % If 'latin', max parameter values
%% Par_info.mu               % If 'normal', mean normal distribution
%% Par_info.cov              % If 'normal', covariance matrix normal
%% Par_info.prior            % Multiplicative or multivariate prior
%%                             if Par_info.initial = 'prior'
%% Par_info.boundhandling    % Boundary handling
%%                             ('reflect'/'bound'/'fold'/'none')
%% ------------------------------------------------------------------------
%%                          DEFAULT VALUES
%% ------------------------------------------------------------------------
%% Par_info.min              % -Inf^{d}
%% Par_info.max              %  Inf^{d}
%% Par_info.boundhandling    % 'none' (unbounded problem)
%% ------------------------------------------------------------------------

%% ########################################################################
%%   Meas_info: Measurement data to compare model output against
%% ########################################################################
%%                             OPTIONAL
%% ------------------------------------------------------------------------
%% Meas_info.Y               % Scalar/vector with calibration data
%% Meas_info.Sigma           % Scalar/vector with corresponding data errors
%% Meas_info.S               % Scalar/vector with summary metrics
%% ------------------------------------------------------------------------

%% ########################################################################
%%   options: Structure with optional settings
%% ########################################################################
%%                             OPTIONAL
%% ------------------------------------------------------------------------
%% options.rho               % ABC distance function ( inline format )
%% options.epsilon           % ABC epsilon value ( scalar or vector )
%% options.DB                % Diagnostic Bayes?
%% options.parallel          % Multi-core computation chains?
%% options.IO                % If parallel, IO writing model?
%% options.modout            % Return model (function) simulations?
%% options.save              % Save DREAM output during the run?
%% options.restart           % Restart run? (only with "save")
%% options.diagnostics       % Compute within-chain diagnostics?
%% options.print             % Output writing to screen (tables/figures)
%% ------------------------------------------------------------------------

%% ------------------------------------------------------------------------
%% If options.modout = 'yes' --> the simulations of the model are stored.
%% in a binary output file during the run. This file is read at the end of
%% the program and returned as variable "fx" (output argument 4 of DREAM).
%% !! This will only be useful if model creates actual simulation !!
%% ------------------------------------------------------------------------